@@ -1,25 +1,25 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.environment.collectors
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     The data collector components for sphinx.environment.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from six import itervalues
+from typing import Dict, List, Set
+
+from docutils import nodes
+
+from sphinx.environment import BuildEnvironment
 
 if False:
     # For type annotation
-    from typing import Dict, List, Set  # NOQA
-    from docutils import nodes  # NOQA
-    from sphinx.sphinx import Sphinx  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.application import Sphinx
 
 
-class EnvironmentCollector(object):
+class EnvironmentCollector:
     """An EnvironmentCollector is a specific data collector from each document.
 
     It gathers data and stores :py:class:`BuildEnvironment
@@ -28,10 +28,9 @@ class EnvironmentCollector(object):
     entries and toctrees, etc.
     """
 
-    listener_ids = None  # type: Dict[unicode, int]
+    listener_ids = None  # type: Dict[str, int]
 
-    def enable(self, app):
-        # type: (Sphinx) -> None
+    def enable(self, app: "Sphinx") -> None:
         assert self.listener_ids is None
         self.listener_ids = {
             'doctree-read':     app.connect('doctree-read', self.process_doc),
@@ -41,43 +40,39 @@ class EnvironmentCollector(object):
             'env-get-outdated': app.connect('env-get-outdated', self.get_outdated_docs),
         }
 
-    def disable(self, app):
-        # type: (Sphinx) -> None
+    def disable(self, app: "Sphinx") -> None:
         assert self.listener_ids is not None
-        for listener_id in itervalues(self.listener_ids):
+        for listener_id in self.listener_ids.values():
             app.disconnect(listener_id)
         self.listener_ids = None
 
-    def clear_doc(self, app, env, docname):
-        # type: (Sphinx, BuildEnvironment, unicode) -> None
+    def clear_doc(self, app: "Sphinx", env: BuildEnvironment, docname: str) -> None:
         """Remove specified data of a document.
 
         This method is called on the removal of the document."""
         raise NotImplementedError
 
-    def merge_other(self, app, env, docnames, other):
-        # type: (Sphinx, BuildEnvironment, Set[unicode], BuildEnvironment) -> None
+    def merge_other(self, app: "Sphinx", env: BuildEnvironment,
+                    docnames: Set[str], other: BuildEnvironment) -> None:
         """Merge in specified data regarding docnames from a different `BuildEnvironment`
         object which coming from a subprocess in parallel builds."""
         raise NotImplementedError
 
-    def process_doc(self, app, doctree):
-        # type: (Sphinx, nodes.Node) -> None
+    def process_doc(self, app: "Sphinx", doctree: nodes.document) -> None:
         """Process a document and gather specific data from it.
 
         This method is called after the document is read."""
         raise NotImplementedError
 
-    def get_updated_docs(self, app, env):
-        # type: (Sphinx, BuildEnvironment) -> List[unicode]
+    def get_updated_docs(self, app: "Sphinx", env: BuildEnvironment) -> List[str]:
         """Return a list of docnames to re-read.
 
         This methods is called after reading the whole of documents (experimental).
         """
         return []
 
-    def get_outdated_docs(self, app, env, added, changed, removed):
-        # type: (Sphinx, BuildEnvironment, unicode, Set[unicode], Set[unicode], Set[unicode]) -> List[unicode]  # NOQA
+    def get_outdated_docs(self, app: "Sphinx", env: BuildEnvironment,
+                          added: Set[str], changed: Set[str], removed: Set[str]) -> List[str]:
         """Return a list of docnames to re-read.
 
         This methods is called before reading the documents.
