@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.extlinks
     ~~~~~~~~~~~~~~~~~~~
@@ -8,11 +7,11 @@
 
     This adds a new config value called ``extlinks`` that is created like this::
 
-       extlinks = {'exmpl': ('http://example.com/%s.html', prefix), ...}
+       extlinks = {'exmpl': ('https://example.invalid/%s.html', prefix), ...}
 
     Now you can use e.g. :exmpl:`foo` in your documents.  This will create a
-    link to ``http://example.com/foo.html``.  The link caption depends on the
-    *prefix* value given:
+    link to ``https://example.invalid/foo.html``.  The link caption depends on
+    the *prefix* value given:
 
     - If it is ``None``, the caption will be the full URL.
     - If it is a string (empty or not), the caption will be the prefix prepended
@@ -20,19 +19,26 @@
 
     You can also give an explicit caption, e.g. :exmpl:`Foo <foo>`.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from six import iteritems
+from typing import Any, Dict, List, Tuple
+
 from docutils import nodes, utils
+from docutils.nodes import Node, system_message
+from docutils.parsers.rst.states import Inliner
 
 import sphinx
+from sphinx.application import Sphinx
 from sphinx.util.nodes import split_explicit_title
+from sphinx.util.typing import RoleFunction
 
 
-def make_link_role(base_url, prefix):
-    def role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
+def make_link_role(base_url: str, prefix: str) -> RoleFunction:
+    def role(typ: str, rawtext: str, text: str, lineno: int,
+             inliner: Inliner, options: Dict = {}, content: List[str] = []
+             ) -> Tuple[List[Node], List[system_message]]:
         text = utils.unescape(text)
         has_explicit_title, title, part = split_explicit_title(text)
         try:
@@ -53,12 +59,12 @@ def make_link_role(base_url, prefix):
     return role
 
 
-def setup_link_roles(app):
-    for name, (base_url, prefix) in iteritems(app.config.extlinks):
+def setup_link_roles(app: Sphinx) -> None:
+    for name, (base_url, prefix) in app.config.extlinks.items():
         app.add_role(name, make_link_role(base_url, prefix))
 
 
-def setup(app):
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_config_value('extlinks', {}, 'env')
     app.connect('builder-inited', setup_link_roles)
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
