@@ -1,33 +1,30 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.linkcode
     ~~~~~~~~~~~~~~~~~~~
 
     Add external links to module code in Python object descriptions.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import Any, Dict, Set
+
 from docutils import nodes
+from docutils.nodes import Node
 
 import sphinx
 from sphinx import addnodes
+from sphinx.application import Sphinx
 from sphinx.errors import SphinxError
 from sphinx.locale import _
 
-if False:
-    # For type annotation
-    from typing import Any, Dict, Set  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
 
 class LinkcodeError(SphinxError):
     category = "linkcode error"
 
 
-def doctree_read(app, doctree):
-    # type: (Sphinx, nodes.Node) -> None
+def doctree_read(app: Sphinx, doctree: Node) -> None:
     env = app.builder.env
 
     resolve_target = getattr(env.config, 'linkcode_resolve', None)
@@ -35,16 +32,16 @@ def doctree_read(app, doctree):
         raise LinkcodeError(
             "Function `linkcode_resolve` is not given in conf.py")
 
-    domain_keys = dict(
-        py=['module', 'fullname'],
-        c=['names'],
-        cpp=['names'],
-        js=['object', 'fullname'],
-    )
+    domain_keys = {
+        'py': ['module', 'fullname'],
+        'c': ['names'],
+        'cpp': ['names'],
+        'js': ['object', 'fullname'],
+    }
 
     for objnode in doctree.traverse(addnodes.desc):
         domain = objnode.get('domain')
-        uris = set()  # type: Set[unicode]
+        uris = set()  # type: Set[str]
         for signode in objnode:
             if not isinstance(signode, addnodes.desc_signature):
                 continue
@@ -70,15 +67,13 @@ def doctree_read(app, doctree):
                 continue
             uris.add(uri)
 
+            inline = nodes.inline('', _('[source]'), classes=['viewcode-link'])
             onlynode = addnodes.only(expr='html')
-            onlynode += nodes.reference('', '', internal=False, refuri=uri)
-            onlynode[0] += nodes.inline('', _('[source]'),
-                                        classes=['viewcode-link'])
+            onlynode += nodes.reference('', '', inline, internal=False, refuri=uri)
             signode += onlynode
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[unicode, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.connect('doctree-read', doctree_read)
     app.add_config_value('linkcode_resolve', None, '')
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
