@@ -9,8 +9,7 @@
 """
 import codecs
 import warnings
-from typing import Any, List, Tuple
-from typing import Type  # for python3.5.1
+from typing import Any, List
 
 from docutils import nodes
 from docutils.core import Publisher
@@ -19,14 +18,12 @@ from docutils.io import FileInput, Input, NullOutput
 from docutils.parsers import Parser
 from docutils.parsers.rst import Parser as RSTParser
 from docutils.readers import standalone
-from docutils.statemachine import StringList, string2lines
 from docutils.transforms import Transform
 from docutils.transforms.references import DanglingReferences
 from docutils.writers import UnfilteredWriter
 
-from sphinx.deprecation import (
-    RemovedInSphinx30Warning, RemovedInSphinx40Warning, deprecated_alias
-)
+from sphinx import addnodes
+from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
 from sphinx.environment import BuildEnvironment
 from sphinx.errors import FiletypeNotFoundError
 from sphinx.transforms import (
@@ -39,11 +36,11 @@ from sphinx.transforms.references import SphinxDomains
 from sphinx.util import logging, get_filetype
 from sphinx.util import UnicodeDecodeErrorHandler
 from sphinx.util.docutils import LoggingReporter
-from sphinx.util.rst import append_epilog, docinfo_re, prepend_prolog
 from sphinx.versioning import UIDTransform
 
 if False:
     # For type annotation
+    from typing import Type  # for python3.5.1
     from sphinx.application import Sphinx
 
 
@@ -84,7 +81,7 @@ class SphinxBaseReader(standalone.Reader):
         self._app = app      # hold application object only for compatibility
         self._env = app.env
 
-    def get_transforms(self) -> List[Type[Transform]]:
+    def get_transforms(self) -> List["Type[Transform]"]:
         transforms = super().get_transforms() + self.transforms
 
         # remove transforms which is not needed for Sphinx
@@ -100,6 +97,7 @@ class SphinxBaseReader(standalone.Reader):
         for logging.
         """
         document = super().new_document()
+        document.__class__ = addnodes.document  # replace the class with patched version
 
         # substitute transformer
         document.transformer = SphinxTransformer(document)
@@ -160,17 +158,6 @@ class SphinxI18nReader(SphinxBaseReader):
             if transform in self.transforms:
                 self.transforms.remove(transform)
 
-    def set_lineno_for_reporter(self, lineno: int) -> None:
-        """Stores the source line number of original text."""
-        warnings.warn('SphinxI18nReader.set_lineno_for_reporter() is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-
-    @property
-    def line(self) -> int:
-        warnings.warn('SphinxI18nReader.line is deprecated.',
-                      RemovedInSphinx30Warning, stacklevel=2)
-        return 0
-
 
 class SphinxDummyWriter(UnfilteredWriter):
     """Dummy writer module used for generating doctree."""
@@ -186,93 +173,13 @@ def SphinxDummySourceClass(source: Any, *args: Any, **kwargs: Any) -> Any:
     return source
 
 
-class SphinxBaseFileInput(FileInput):
-    """A base class of SphinxFileInput.
-
-    It supports to replace unknown Unicode characters to '?'.
-    """
-
-    def __init__(self, app: "Sphinx", env: BuildEnvironment,
-                 *args: Any, **kwargs: Any) -> None:
-        self.app = app
-        self.env = env
-
-        warnings.warn('%s is deprecated.' % self.__class__.__name__,
-                      RemovedInSphinx30Warning, stacklevel=2)
-
-        kwargs['error_handler'] = 'sphinx'  # py3: handle error on open.
-        super().__init__(*args, **kwargs)
-
-    def warn_and_replace(self, error: Any) -> Tuple:
-        return UnicodeDecodeErrorHandler(self.env.docname)(error)
-
-
 class SphinxFileInput(FileInput):
     """A basic FileInput for Sphinx."""
-    supported = ('*',)  # RemovedInSphinx30Warning
-
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         kwargs['error_handler'] = 'sphinx'
         super().__init__(*args, **kwargs)
 
 
-class SphinxRSTFileInput(SphinxBaseFileInput):
-    """A reST FileInput for Sphinx.
-
-    This FileInput automatically prepends and appends text by :confval:`rst_prolog` and
-    :confval:`rst_epilog`.
-
-    .. important::
-
-       This FileInput uses an instance of ``StringList`` as a return value of ``read()``
-       method to indicate original source filename and line numbers after prepending and
-       appending.
-       For that reason, ``sphinx.parsers.RSTParser`` should be used with this to parse
-       a content correctly.
-    """
-    supported = ('restructuredtext',)
-
-    def prepend_prolog(self, text: StringList, prolog: str) -> None:
-        docinfo = self.count_docinfo_lines(text)
-        if docinfo:
-            # insert a blank line after docinfo
-            text.insert(docinfo, '', '<generated>', 0)
-            docinfo += 1
-
-        # insert prolog (after docinfo if exists)
-        for lineno, line in enumerate(prolog.splitlines()):
-            text.insert(docinfo + lineno, line, '<rst_prolog>', lineno)
-
-        text.insert(docinfo + lineno + 1, '', '<generated>', 0)
-
-    def append_epilog(self, text: StringList, epilog: str) -> None:
-        # append a blank line and rst_epilog
-        text.append('', '<generated>', 0)
-        for lineno, line in enumerate(epilog.splitlines()):
-            text.append(line, '<rst_epilog>', lineno)
-
-    def read(self) -> StringList:  # type: ignore
-        inputstring = super().read()
-        lines = string2lines(inputstring, convert_whitespace=True)
-        content = StringList()
-        for lineno, line in enumerate(lines):
-            content.append(line, self.source_path, lineno)
-
-        prepend_prolog(content, self.env.config.rst_prolog)
-        append_epilog(content, self.env.config.rst_epilog)
-
-        return content
-
-    def count_docinfo_lines(self, content: StringList) -> int:
-        if len(content) == 0:
-            return 0
-        else:
-            for lineno, line in enumerate(content.data):
-                if not docinfo_re.match(line):
-                    break
-            return lineno
-
-
 def read_doc(app: "Sphinx", env: BuildEnvironment, filename: str) -> nodes.document:
     """Parse a document and convert to doctree."""
     # set up error_handler for the target document
@@ -322,4 +229,8 @@ deprecated_alias('sphinx.io',
                      'FiletypeNotFoundError': FiletypeNotFoundError,
                      'get_filetype': get_filetype,
                  },
-                 RemovedInSphinx40Warning)
+                 RemovedInSphinx40Warning,
+                 {
+                     'FiletypeNotFoundError': 'sphinx.errors.FiletypeNotFoundError',
+                     'get_filetype': 'sphinx.util.get_filetype',
+                 })
