@@ -14,20 +14,59 @@ import sys
 from collections import namedtuple
 from io import StringIO
 from subprocess import PIPE
-from typing import Any, Dict
+from typing import Any, Callable, Dict, Generator, Tuple
 
 import pytest
 
-from . import util
+from sphinx.testing import util
+from sphinx.testing.util import SphinxTestApp, SphinxTestAppWrapperForSkipBuilding
+
+
+DEFAULT_ENABLED_MARKERS = [
+    (
+        'sphinx(builder, testroot=None, freshenv=False, confoverrides=None, tags=None,'
+        ' docutilsconf=None, parallel=0): arguments to initialize the sphinx test application.'
+    ),
+    'test_params(shared_result=...): test parameters.',
+]
+
+
+def pytest_configure(config):
+    # register custom markers
+    for marker in DEFAULT_ENABLED_MARKERS:
+        config.addinivalue_line('markers', marker)
 
 
 @pytest.fixture(scope='session')
-def rootdir() -> None:
+def rootdir() -> str:
     return None
 
 
+class SharedResult:
+    cache = {}  # type: Dict[str, Dict[str, str]]
+
+    def store(self, key: str, app_: SphinxTestApp) -> Any:
+        if key in self.cache:
+            return
+        data = {
+            'status': app_._status.getvalue(),
+            'warning': app_._warning.getvalue(),
+        }
+        self.cache[key] = data
+
+    def restore(self, key: str) -> Dict[str, StringIO]:
+        if key not in self.cache:
+            return {}
+        data = self.cache[key]
+        return {
+            'status': StringIO(data['status']),
+            'warning': StringIO(data['warning']),
+        }
+
+
 @pytest.fixture
-def app_params(request, test_params, shared_result, sphinx_test_tempdir, rootdir):
+def app_params(request: Any, test_params: Dict, shared_result: SharedResult,
+               sphinx_test_tempdir: str, rootdir: str) -> Tuple[Dict, Dict]:
     """
     parameters that is specified by 'pytest.mark.sphinx' for
     sphinx.application.Sphinx initialization
@@ -40,7 +79,7 @@ def app_params(request, test_params, shared_result, sphinx_test_tempdir, rootdir
     else:
         markers = request.node.get_marker("sphinx")
     pargs = {}
-    kwargs = {}  # type: Dict[str, str]
+    kwargs = {}  # type: Dict[str, Any]
 
     if markers is not None:
         # to avoid stacking positional args
@@ -55,7 +94,7 @@ def app_params(request, test_params, shared_result, sphinx_test_tempdir, rootdir
 
     if test_params['shared_result']:
         if 'srcdir' in kwargs:
-            raise pytest.Exception('You can not spcify shared_result and '
+            raise pytest.Exception('You can not specify shared_result and '
                                    'srcdir in same time.')
         kwargs['srcdir'] = test_params['shared_result']
         restore = shared_result.restore(test_params['shared_result'])
@@ -75,7 +114,7 @@ def app_params(request, test_params, shared_result, sphinx_test_tempdir, rootdir
 
 
 @pytest.fixture
-def test_params(request):
+def test_params(request: Any) -> Dict:
     """
     test parameters that is specified by 'pytest.mark.test_params'
 
@@ -102,7 +141,8 @@ def test_params(request):
 
 
 @pytest.fixture(scope='function')
-def app(test_params, app_params, make_app, shared_result):
+def app(test_params: Dict, app_params: Tuple[Dict, Dict], make_app: Callable,
+        shared_result: SharedResult) -> Generator[SphinxTestApp, None, None]:
     """
     provides sphinx.application.Sphinx object
     """
@@ -122,7 +162,7 @@ def app(test_params, app_params, make_app, shared_result):
 
 
 @pytest.fixture(scope='function')
-def status(app):
+def status(app: SphinxTestApp) -> StringIO:
     """
     compat for testing with previous @with_app decorator
     """
@@ -130,7 +170,7 @@ def status(app):
 
 
 @pytest.fixture(scope='function')
-def warning(app):
+def warning(app: SphinxTestApp) -> StringIO:
     """
     compat for testing with previous @with_app decorator
     """
@@ -138,7 +178,7 @@ def warning(app):
 
 
 @pytest.fixture()
-def make_app(test_params, monkeypatch):
+def make_app(test_params: Dict, monkeypatch: Any) -> Generator[Callable, None, None]:
     """
     provides make_app function to initialize SphinxTestApp instance.
     if you want to initialize 'app' in your test function. please use this
@@ -153,10 +193,10 @@ def make_app(test_params, monkeypatch):
         status, warning = StringIO(), StringIO()
         kwargs.setdefault('status', status)
         kwargs.setdefault('warning', warning)
-        app_ = util.SphinxTestApp(*args, **kwargs)  # type: Any
+        app_ = SphinxTestApp(*args, **kwargs)  # type: Any
         apps.append(app_)
         if test_params['shared_result']:
-            app_ = util.SphinxTestAppWrapperForSkipBuilding(app_)
+            app_ = SphinxTestAppWrapperForSkipBuilding(app_)
         return app_
     yield make
 
@@ -165,40 +205,18 @@ def make_app(test_params, monkeypatch):
         app_.cleanup()
 
 
-class SharedResult:
-    cache = {}  # type: Dict[str, Dict[str, str]]
-
-    def store(self, key, app_):
-        if key in self.cache:
-            return
-        data = {
-            'status': app_._status.getvalue(),
-            'warning': app_._warning.getvalue(),
-        }
-        self.cache[key] = data
-
-    def restore(self, key):
-        if key not in self.cache:
-            return {}
-        data = self.cache[key]
-        return {
-            'status': StringIO(data['status']),
-            'warning': StringIO(data['warning']),
-        }
-
-
 @pytest.fixture
-def shared_result():
+def shared_result() -> SharedResult:
     return SharedResult()
 
 
 @pytest.fixture(scope='module', autouse=True)
-def _shared_result_cache():
+def _shared_result_cache() -> None:
     SharedResult.cache.clear()
 
 
 @pytest.fixture
-def if_graphviz_found(app):
+def if_graphviz_found(app: SphinxTestApp) -> None:
     """
     The test will be skipped when using 'if_graphviz_found' fixture and graphviz
     dot command is not found.
@@ -215,7 +233,7 @@ def if_graphviz_found(app):
 
 
 @pytest.fixture(scope='session')
-def sphinx_test_tempdir(tmpdir_factory):
+def sphinx_test_tempdir(tmpdir_factory: Any) -> "util.path":
     """
     temporary directory that wrapped with `path` class.
     """
@@ -227,7 +245,7 @@ def sphinx_test_tempdir(tmpdir_factory):
 
 
 @pytest.fixture
-def tempdir(tmpdir):
+def tempdir(tmpdir: str) -> "util.path":
     """
     temporary directory that wrapped with `path` class.
     this fixture is for compat with old test implementation.
