@@ -1,5 +1,9 @@
+from functools import wraps
+
+
 def deco1(func):
     """docstring for deco1"""
+    @wraps(func)
     def wrapper():
         return func()
 
@@ -14,3 +18,14 @@ def deco2(condition, message):
 
         return wrapper
     return decorator
+
+
+@deco1
+def foo(name=None, age=None):
+    pass
+
+
+class Bar:
+    @deco1
+    def meth(self, name=None, age=None):
+        pass
