@@ -1,10 +1,8 @@
-# -*- coding: utf-8 -*-
-
 project = 'Sphinx intl <Tests>'
 source_suffix = '.txt'
 keep_warnings = True
 templates_path = ['_templates']
-html_additional_pages = {'index': 'index.html'}
+html_additional_pages = {'contents': 'contents.html'}
 release = version = '2013.120'
 gettext_additional_targets = ['index']
 exclude_patterns = ['_build']
