@@ -17,7 +17,7 @@ def test_build(app):
     app.build()
 
     # TODO: Use better checking of html content
-    htmltext = (app.outdir / 'changes.html').text()
+    htmltext = (app.outdir / 'changes.html').read_text()
     assert 'New in version 0.6: Some funny stuff.' in htmltext
     assert 'Changed in version 0.6: Even more funny stuff.' in htmltext
     assert 'Deprecated since version 0.6: Boring stuff.' in htmltext
@@ -28,7 +28,7 @@ def test_build(app):
     assert path_html in htmltext
 
     malloc_html = (
-        '<b>Test_Malloc</b>: <i>changed:</i> Changed in version 0.6:'
+        '<b>void *Test_Malloc(size_t n)</b>: <i>changed:</i> Changed in version 0.6:'
         ' Can now be replaced with a different allocator.</a>')
     assert malloc_html in htmltext
 
