@@ -14,7 +14,12 @@ import pytest
 from docutils import nodes
 
 from sphinx import addnodes
+from sphinx.addnodes import (
+    desc, desc_annotation, desc_content, desc_name,
+    desc_parameter, desc_parameterlist, desc_signature
+)
 from sphinx.domains.javascript import JavaScriptDomain
+from sphinx.testing import restructuredtext
 from sphinx.testing.util import assert_node
 
 
@@ -86,22 +91,22 @@ def test_domain_js_objects(app, status, warning):
     assert 'module_b.submodule' in modules
     assert 'module_b.submodule' in objects
 
-    assert objects['module_a.submodule.ModTopLevel'] == ('module', 'class')
-    assert objects['module_a.submodule.ModTopLevel.mod_child_1'] == ('module', 'method')
-    assert objects['module_a.submodule.ModTopLevel.mod_child_2'] == ('module', 'method')
-    assert objects['module_b.submodule.ModTopLevel'] == ('module', 'class')
+    assert objects['module_a.submodule.ModTopLevel'][2] == 'class'
+    assert objects['module_a.submodule.ModTopLevel.mod_child_1'][2] == 'method'
+    assert objects['module_a.submodule.ModTopLevel.mod_child_2'][2] == 'method'
+    assert objects['module_b.submodule.ModTopLevel'][2] == 'class'
 
-    assert objects['TopLevel'] == ('roles', 'class')
-    assert objects['top_level'] == ('roles', 'function')
-    assert objects['NestedParentA'] == ('roles', 'class')
-    assert objects['NestedParentA.child_1'] == ('roles', 'function')
-    assert objects['NestedParentA.any_child'] == ('roles', 'function')
-    assert objects['NestedParentA.NestedChildA'] == ('roles', 'class')
-    assert objects['NestedParentA.NestedChildA.subchild_1'] == ('roles', 'function')
-    assert objects['NestedParentA.NestedChildA.subchild_2'] == ('roles', 'function')
-    assert objects['NestedParentA.child_2'] == ('roles', 'function')
-    assert objects['NestedParentB'] == ('roles', 'class')
-    assert objects['NestedParentB.child_1'] == ('roles', 'function')
+    assert objects['TopLevel'][2] == 'class'
+    assert objects['top_level'][2] == 'function'
+    assert objects['NestedParentA'][2] == 'class'
+    assert objects['NestedParentA.child_1'][2] == 'function'
+    assert objects['NestedParentA.any_child'][2] == 'function'
+    assert objects['NestedParentA.NestedChildA'][2] == 'class'
+    assert objects['NestedParentA.NestedChildA.subchild_1'][2] == 'function'
+    assert objects['NestedParentA.NestedChildA.subchild_2'][2] == 'function'
+    assert objects['NestedParentA.child_2'][2] == 'function'
+    assert objects['NestedParentB'][2] == 'class'
+    assert objects['NestedParentB.child_1'][2] == 'function'
 
 
 @pytest.mark.sphinx('dummy', testroot='domain-js')
@@ -115,21 +120,28 @@ def test_domain_js_find_obj(app, status, warning):
 
     assert (find_obj(None, None, 'NONEXISTANT', 'class') == (None, None))
     assert (find_obj(None, None, 'NestedParentA', 'class') ==
-            ('NestedParentA', ('roles', 'class')))
+            ('NestedParentA', ('roles', 'NestedParentA', 'class')))
     assert (find_obj(None, None, 'NestedParentA.NestedChildA', 'class') ==
-            ('NestedParentA.NestedChildA', ('roles', 'class')))
+            ('NestedParentA.NestedChildA',
+             ('roles', 'NestedParentA.NestedChildA', 'class')))
     assert (find_obj(None, 'NestedParentA', 'NestedChildA', 'class') ==
-            ('NestedParentA.NestedChildA', ('roles', 'class')))
+            ('NestedParentA.NestedChildA',
+             ('roles', 'NestedParentA.NestedChildA', 'class')))
     assert (find_obj(None, None, 'NestedParentA.NestedChildA.subchild_1', 'func') ==
-            ('NestedParentA.NestedChildA.subchild_1', ('roles', 'function')))
+            ('NestedParentA.NestedChildA.subchild_1',
+             ('roles', 'NestedParentA.NestedChildA.subchild_1', 'function')))
     assert (find_obj(None, 'NestedParentA', 'NestedChildA.subchild_1', 'func') ==
-            ('NestedParentA.NestedChildA.subchild_1', ('roles', 'function')))
+            ('NestedParentA.NestedChildA.subchild_1',
+             ('roles', 'NestedParentA.NestedChildA.subchild_1', 'function')))
     assert (find_obj(None, 'NestedParentA.NestedChildA', 'subchild_1', 'func') ==
-            ('NestedParentA.NestedChildA.subchild_1', ('roles', 'function')))
+            ('NestedParentA.NestedChildA.subchild_1',
+             ('roles', 'NestedParentA.NestedChildA.subchild_1', 'function')))
     assert (find_obj('module_a.submodule', 'ModTopLevel', 'mod_child_2', 'meth') ==
-            ('module_a.submodule.ModTopLevel.mod_child_2', ('module', 'method')))
+            ('module_a.submodule.ModTopLevel.mod_child_2',
+             ('module', 'module_a.submodule.ModTopLevel.mod_child_2', 'method')))
     assert (find_obj('module_b.submodule', 'ModTopLevel', 'module_a.submodule', 'mod') ==
-            ('module_a.submodule', ('module', 'module')))
+            ('module_a.submodule',
+             ('module', 'module-module_a.submodule', 'module')))
 
 
 def test_get_full_qualified_name():
@@ -158,3 +170,61 @@ def test_get_full_qualified_name():
     kwargs = {'js:module': 'module1', 'js:object': 'Class'}
     node = nodes.reference(reftarget='func', **kwargs)
     assert domain.get_full_qualified_name(node) == 'module1.Class.func'
+
+
+def test_js_module(app):
+    text = ".. js:module:: sphinx"
+    doctree = restructuredtext.parse(app, text)
+    assert_node(doctree, (nodes.target,
+                          addnodes.index))
+    assert_node(doctree[0], nodes.target, ids=["module-sphinx"])
+    assert_node(doctree[1], addnodes.index,
+                entries=[("single", "sphinx (module)", "module-sphinx", "", None)])
+
+
+def test_js_function(app):
+    text = ".. js:function:: sum(a, b)"
+    doctree = restructuredtext.parse(app, text)
+    assert_node(doctree, (addnodes.index,
+                          [desc, ([desc_signature, ([desc_name, "sum"],
+                                                    desc_parameterlist)],
+                                  [desc_content, ()])]))
+    assert_node(doctree[1][0][1], [desc_parameterlist, ([desc_parameter, "a"],
+                                                        [desc_parameter, "b"])])
+    assert_node(doctree[0], addnodes.index,
+                entries=[("single", "sum() (built-in function)", "sum", "", None)])
+    assert_node(doctree[1], addnodes.desc, domain="js", objtype="function", noindex=False)
+
+
+def test_js_class(app):
+    text = ".. js:class:: Application"
+    doctree = restructuredtext.parse(app, text)
+    assert_node(doctree, (addnodes.index,
+                          [desc, ([desc_signature, ([desc_annotation, "class "],
+                                                    [desc_name, "Application"],
+                                                    [desc_parameterlist, ()])],
+                                  [desc_content, ()])]))
+    assert_node(doctree[0], addnodes.index,
+                entries=[("single", "Application() (class)", "Application", "", None)])
+    assert_node(doctree[1], addnodes.desc, domain="js", objtype="class", noindex=False)
+
+
+def test_js_data(app):
+    text = ".. js:data:: name"
+    doctree = restructuredtext.parse(app, text)
+    assert_node(doctree, (addnodes.index,
+                          [desc, ([desc_signature, desc_name, "name"],
+                                  [desc_content, ()])]))
+    assert_node(doctree[0], addnodes.index,
+                entries=[("single", "name (global variable or constant)", "name", "", None)])
+    assert_node(doctree[1], addnodes.desc, domain="js", objtype="data", noindex=False)
+
+
+def test_noindexentry(app):
+    text = (".. js:function:: f()\n"
+            ".. js:function:: g()\n"
+            "   :noindexentry:\n")
+    doctree = restructuredtext.parse(app, text)
+    assert_node(doctree, (addnodes.index, desc, addnodes.index, desc))
+    assert_node(doctree[0], addnodes.index, entries=[('single', 'f() (built-in function)', 'f', '', None)])
+    assert_node(doctree[2], addnodes.index, entries=[])
