@@ -14,6 +14,6 @@ import pytest
 @pytest.mark.sphinx('text', testroot='ext-ifconfig')
 def test_ifconfig(app, status, warning):
     app.builder.build_all()
-    result = (app.outdir / 'index.txt').text()
+    result = (app.outdir / 'index.txt').read_text()
     assert 'spam' in result
     assert 'ham' not in result
