@@ -18,7 +18,7 @@ import sphinx
 from sphinx.errors import ExtensionError, PycodeError
 from sphinx.testing.util import strip_escseq
 from sphinx.util import (
-    SkipProgressMessage, display_chunk, encode_uri, ensuredir, get_module_source,
+    SkipProgressMessage, display_chunk, encode_uri, ensuredir,
     import_object, parselinenos, progress_message, status_iterator, xmlname_checker
 )
 from sphinx.util import logging
@@ -61,16 +61,6 @@ def test_display_chunk():
     assert display_chunk(('hello', 'sphinx', 'world')) == 'hello .. world'
 
 
-def test_get_module_source():
-    assert get_module_source('sphinx') == ('file', sphinx.__file__)
-
-    # failed to obtain source information from builtin modules
-    with pytest.raises(PycodeError):
-        get_module_source('builtins')
-    with pytest.raises(PycodeError):
-        get_module_source('itertools')
-
-
 def test_import_object():
     module = import_object('sphinx')
     assert module.__name__ == 'sphinx'
