@@ -1,5 +1,6 @@
+from inspect import Signature
 from numbers import Integral
-from typing import Any, List, TypeVar, Union, Callable, Tuple, Optional
+from typing import Any, Dict, List, TypeVar, Union, Callable, Tuple, Optional
 
 
 def f0(x: int, y: Integral) -> None:
@@ -80,6 +81,30 @@ def f15(x: "Unknown", y: "int") -> Any:
     pass
 
 
+def f16(arg1, arg2, *, arg3=None, arg4=None):
+    pass
+
+
+def f17(*, arg3, arg4):
+    pass
+
+
+def f18(self, arg1: Union[int, Tuple] = 10) -> List[Dict]:
+    pass
+
+
+def f19(*args: int, **kwargs: str):
+    pass
+
+
+def f20() -> Optional[Union[int, str]]:
+    pass
+
+
+def f21(arg1='whatever', arg2=Signature.empty):
+    pass
+
+
 class Node:
     def __init__(self, parent: Optional['Node']) -> None:
         pass
